
 /* simple voids to work with the two body problem kepler elements */

 /* unit unforkep */

#include <math.h>

#include "unforkep.h"
#include "unforfun.h"

 /* for minor planet
   MeanMotion   : mean motion  n  in radian per day
   semi-major axis in AU
   Sqr(GaussConst)  in AU**3/(day**2*SolarMass)
   from the formula
   n^2*a^3=k^2
   or  n=k/sqrt(a^3) where  k = 0.01720209895 - Gauss const */

 double fromaxistomot ( double a )     /* semimajor axis in AU */
 {                                     /* simple function */
   return (0.01720209895/sqrt(a*a*a)); /* in radian per day */
 }

 /* kepler equation   M=E-e*sin(E)
   to solve with the use newton iteration method
   if  fi(x)=0  then iteration may be possible
   x_(n+1)=f(x_n)  where  f(x)=x-fi(x)/(d/dx(fi(x))) */

 double keplerequation ( double eleme , double meananom )
 {
   int iter ;
   double eccanom,cura ;

   cura = meananom+eleme*sin(meananom); /* in radian */
   for ( iter = 1 ; iter <= 5 ; iter++ )
     {
       eccanom = cura-(cura-eleme*sin(cura)-meananom)/(1-eleme*cos(cura));
       cura = eccanom;
     }
   return (eccanom); /* eccentric anomaly in radian */
 }

 /* to change
    from heliocentric but fixed equator
    descart position and velosity
    to   heliocentric ecliptic
    Kepler osculating elements by using obliqiuty value

   input    equatorial system
   start    moment of time in modified julian day
   position pos x,y,z in AU      for moment tstart
   velosity vel vx,vy,vz AU/day  for moment tstart
   finish   moment of time for Kepler elements

   gaussconst = 0.01720209895 Gauss gravitational constant
   sineps   is   sinus of obliquity
   coseps   is cosinus of obliquity

   output   ecliptical system
   posvar 1,2,3 : a in AU, e,  inclination in degree
   angvar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly in degree */

 void fromcarttokepler ( double start ,                        /*  input */
                         double poscur[4] , double velcur[4] , /*  input */
                         double finish ,                       /*  input */
                         double posvar[4] , double angvar[4] ) /* output */
 {
   int i ;          /* Gauss gravitation const in sqr */
   double factormu = sqr(0.01720209895);
   double sineps = sin(84381.448*(M_PI/(180.0*3600.0))) ;
   double coseps = cos(84381.448*(M_PI/(180.0*3600.0))) ;
   double pos[4],vel[4] ;
   double h,c,l,r,v2,q ;
   double c1,c2,c3,l1,l2,l3 ;
   double node,argp,a,e,rincl,trueanom ;
   double eccanom,meananom,si,ci,su,cu,sn,cn,sp,cp,sv,cv,se,ce ;

   /* input position poscur velosity velcur fixed equator
      change to ecliptic by using obliquity R_x(+\epsilon) */

   pos[1] = +poscur[1];                          /* x ecliptic AU */
   pos[2] = +coseps*poscur[2]+sineps*poscur[3]; /* y ecliptic AU */
   pos[3] = -sineps*poscur[2]+coseps*poscur[3]; /* z ecliptic AU */
                              /* velosity */
   /* from equator to ecliptic by using obliquity */
   vel[1] = +velcur[1]; /* R_x(+\epsilon) */    /* v_x equator */
   vel[2] = +coseps*velcur[2]+sineps*velcur[3]; /* v_y equator */
   vel[3] = -sineps*velcur[2]+coseps*velcur[3]; /* v_z equator */

   r = sqrt(pos[1]*pos[1]+pos[2]*pos[2]+pos[3]*pos[3]); /* in AU */

   v2 = vel[1]*vel[1]+vel[2]*vel[2]+vel[3]*vel[3]; /* in (AU/day)^2 */

   q = factormu/r; /* fm_Sun in (AU^3/day^2)/AU */

   h = v2-2*q; /* (AU/day)^2 energy integral */

   c1 = pos[2]*vel[3]-pos[3]*vel[2]; /* three momentum integrals */
   c2 = pos[3]*vel[1]-pos[1]*vel[3];
   c3 = pos[1]*vel[2]-pos[2]*vel[1];
   c = sqrt(c1*c1+c2*c2+c3*c3);

   l1 = -q*pos[1]+vel[2]*c3-vel[3]*c2; /* three integrals */
   l2 = -q*pos[2]+vel[3]*c1-vel[1]*c3;
   l3 = -q*pos[3]+vel[1]*c2-vel[2]*c1;
   l = sqrt(l1*l1+l2*l2+l3*l3);        /* by Laplas */

   a = -factormu/h; /* in AU semi-major axis */

   e = l/factormu; /* eccentricity */
                   /* for the inclination */
   ci = c3/c;
   si = sqrt(fabs(1-ci*ci));
   if  ( si < 1.0e-11 )
     {
       si = 1.0e-11;
       rincl = 0.0; /* singularity inclination = 0 */
     }
     else
       rincl = datan2(si,ci); /* inclination in radian */
                         /* for the ascending node */
   sn = c1/(c*si);
   cn = -c2/(c*si);
   node = datan2(sn,cn); /* node in radian */

   sp = l3/(l*si);
   cp = (l1/l)*cn+(l2/l)*sn;
   argp = datan2(sp,cp); /* argument of perigei in radian */

   q = 1.0/r;
   su = q*pos[3]/si;
   cu = q*pos[1]*cn+q*pos[2]*sn;
   sv = su*cp-cu*sp;
   cv = cu*cp+su*sp; /* argument of latitude u = v + omega */

   trueanom = datan2(sv,cv); /* true anomaly in radian */

   q = 1.0/(1.0+e*cv);
   se = sqrt(1.0-e*e)*sv*q;
   ce = (cv+e)*q;
   eccanom = trueanom+atan((se*cv-ce*sv)/(ce*cv+se*sv));
   meananom = eccanom-e*se; /* M = E - e * sin(E) in radian */

   posvar[1] = a; /* in AU */
   posvar[2] = e;
   posvar[3] = (180.0/M_PI)*rincl; /* in degree */
   angvar[1] = (180.0/M_PI)*node; /* in degree */
   angvar[2] = (180.0/M_PI)*argp; /* in degree */
   angvar[3] = (180.0/M_PI)*(meananom
                +fromaxistomot(a)*(finish-start)); /* in degree */

   for (  i = 1 ;  i <= 3 ; i++ )
     while  ( angvar[i] < 0.0 )  angvar[i] = angvar[i]+360.0;

 }

 /* to change from
    from heliocentric ecliptic
    Kepler osculating elements by using obliqiuty value
    to   descart position and velosity on fixed equator

   factormu is Gauss gravitation const in sqr AU^3/day^2
   sineps   is   sinus of obliquity
   coseps   is cosinus of obliquity

   input
   start  moment in modified julian day as epoch of the elements
   posvar 1,2,3 : a in AU, e,  inclination in degree
   angvar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly un degree
   finish moment in modified julian day
          for the result position velosity

   output
   position pos 1,2,3  : x,y,z in AU 
   velosity vel 1,2,3  : vx,vy,vz in AU/day */

 void fromkeplertocart ( double start ,                  /*  input */
                         double posvar[4] ,              /*  input */
                         double angvar[4] ,              /*  input */
                         double finish ,                 /*  input */
                         double pos[4] , double vel[4] ) /* output */
 {
   double q,vr,vn,eccanom,meananom,trueanom,arglatitude,ascendingnode ;
   double r,xr,yr,zr,xv,yv,zv,elema,eleme,cosinc,sininc ;
   double cose,sine,cosv,sinv,cosu,sinu,cosnode,sinnode ;
   double poscur[4],velcur[4] ;
   double sineps = sin(84381.448*(M_PI/(180.0*3600.0))) ;
   double coseps = cos(84381.448*(M_PI/(180.0*3600.0))) ;
   double factormu = sqr(0.01720209895) ;
   double grarad = M_PI/180.0 ;
                       /* elements ecliptical */
   elema = posvar[1];  /* semimajor axis */

   eleme = posvar[2];  /* eccentricity */

   cosinc = cos(grarad*posvar[3]);
   sininc = sin(grarad*posvar[3]);

   meananom = grarad*angvar[3]  /* to radian the mean anomaly M */
            +fromaxistomot(elema)*(finish-start); /* with mean motion */
                                /* for the eccentric  anomaly E */
   eccanom = keplerequation(eleme,meananom); /* E in readian */

   sine = sin(eccanom);
   cose = cos(eccanom);
                                /* for the true anomaly v */
   sinv = sqrt(1-eleme*eleme)*sine/(1-eleme*cose);
   cosv = (cose-eleme)/(1-eleme*cose);
   trueanom = eccanom+atan((sinv*cose-cosv*sine)/(cosv*cose+sinv*sine));
                               /* the argument of latitude u radian */
   arglatitude = trueanom+grarad*angvar[2]; /* in radian */
   sinu = sin(arglatitude);
   cosu = cos(arglatitude);
 
   ascendingnode = grarad*angvar[1]; /* the ascending node in radian */
   sinnode = sin(ascendingnode);
   cosnode = cos(ascendingnode);

   r = elema*(1-eleme*cose); /* in AU range modul */
   xr = +cosu*cosnode-sinu*sinnode*cosinc; /* x/r */
   yr = +cosu*sinnode+sinu*cosnode*cosinc; /* y/r */
   zr = +sinu*sininc;                      /* z/r */

   xv = -sinu*cosnode-cosu*sinnode*cosinc;
   yv = -sinu*sinnode+cosu*cosnode*cosinc;
   zv = +cosu*sininc;

   q = sqrt(factormu/(elema*(1-eleme*eleme))); /* AU/day */

   vr = q*eleme*sinv;      /* radial  velosity V_r AU/day */

   vn = q*(1+eleme*cosv);  /* tangent valosity V_n AU/day */
                           /* position ecliptical */
   poscur[1] = r*xr;          /* x AU */
   poscur[2] = r*yr;          /* y AU */
   poscur[3] = r*zr;          /* z AU */
                           /* velosity ecliptical */
   velcur[1] = vr*xr+vn*xv;   /* v_x AU/day */
   velcur[2] = vr*yr+vn*yv;   /* v_y AU/day */
   velcur[3] = vr*zr+vn*zv;   /* v_z AU/day */
                              /* position */
   /* to fixed equator by using obliquity */
   pos[1] = +poscur[1]; /* R_x(-\epsilon) */    /* x AU equator */
   pos[2] = +coseps*poscur[2]-sineps*poscur[3]; /* y AU equator */
   pos[3] = +sineps*poscur[2]+coseps*poscur[3]; /* z AU equator */
                              /* velosity */
   /* to fixed equator by using obliquity */
   vel[1] = +velcur[1]; /* R_x(-\epsilon) */    /* v_x equator */
   vel[2] = +coseps*velcur[2]-sineps*velcur[3]; /* v_y equator */
   vel[3] = +sineps*velcur[2]+coseps*velcur[3]; /* v_z equator */
   /* end of procedure */
 }

